import 'package:flutter/cupertino.dart';
import 'package:flutter_mylibrary/base/request_info.dart';
import 'package:flutter_mylibrary/config/const.dart';
import 'package:flutter_mylibrary/flutter_mylibrary.dart';
import 'package:flutter_mylibrary/http/easy_dio.dart';
import 'package:flutter_mylibrary/http/exception/http_error.dart';
import 'package:flutter_mylibrary/http/http_data.dart';
import 'package:flutter_mylibrary/utils/utils.dart';
import 'package:flutter_mylibrary/widget/widget_template.dart';

abstract class BaseViewState{

  /// 防止页面销毁后,异步任务才完成,导致报错
  bool disposed = false;

  /// 当前的页面状态,默认为busy,可在viewModel的构造方法中指定;
  ViewState viewState = ViewState.busy;

  EasyRefreshController _refreshController = EasyRefreshController();

  set refreshController(EasyRefreshController easyRefreshController){
    this._refreshController = easyRefreshController;
  }

  ///获取请求的数据，一般用于简单的页面请求
  RequestInfo getRequestInfo();

  RequestInfo beforeRequest(RequestInfo requestInfo){
    return requestInfo;
  }

  RequestInfo requestInfo;

  void doOther(){}

  void refreshData(){}

  EasyRefreshController get refreshController => _refreshController;

  bool isInit = false;


  ///重新请求当报错时
  void requestWhenError(){
    setBusy();
    refreshData();
  }

  ///---------------------------------------------页面状态------------------------------------------------------

  HttpError httpError;

  bool get isBusy => viewState == ViewState.busy;

  bool get isIdle => viewState == ViewState.idle;

  bool get isError => viewState == ViewState.error;



  ///---------------------------------------------设置页面状态------------------------------------------------------


  void setIdle();

  void setBusy();

  void setError(HttpError httpError);

  void onError(HttpError httpError) {}


  ///---------------------------------------------通用请求方法------------------------------------------------------

  ///通常有下拉刷新请求，第一页之后不会再设置页面错误状态
  Future<HttpData> request(void handleHttpData(HttpData data), {bool isSetError = true}) async{

    requestInfo = beforeRequest(requestInfo??getRequestInfo());

    doOther();

    HttpData _httpData = await doRequest(requestInfo);

    var data = _httpData.data;

    if(isEmpty(_httpData.httpError)){
      if(isNotEmpty(data)){
        handleHttpData(_httpData);
      }else{
        _httpData.httpError = HttpError.emptyDataError(data);
      }
      return _httpData;
    }else{
      if(isSetError){
        setError(_httpData.httpError);
      }
      return _httpData;
    }
  }


  Future<HttpData> doRequest(RequestInfo requestInfo) async{
    HttpData httpData = await EasyDio().request(
      requestInfo?.method,
      requestInfo?.url,
      data: requestInfo?.data,
      publicMethod: requestInfo?.publicMethod,
    );

    try{
      var data = httpData.data;
      if(data is List){
        httpData.data = data.map((e) => requestInfo?.handleRes(e)).toList();
      }else if(data is Map){
        httpData.data = requestInfo?.handleRes(data);
      }
      if(isNotEmpty(requestInfo.validateError) && isEmpty(httpData.httpError)){
        httpData.httpError = requestInfo.validateError(httpData.data);
      }
    }catch(e){
      httpData.httpError =  HttpError.parseDataError(e);
    }


    return httpData;
  }


///------------------------------------------------base下拉刷新控件----------------------------------------------------------

  Widget buildBaseEasyRefresh<T>({
    T data,
    ScrollController scrollController,
    Widget footWidget,
    Widget emptyWidget,
    ViewStateModelWidgetListBuilder<T> widgetListBuilder,
    ViewStateModelWidgetBuilder<T> widgetBuilder,
    Widget busyWidget,
    bool showErrorWidget = true,
    bool showEmptyWidget = true,
    bool showBusyWidget = true,
    Function onLoad,
    Function onRefresh,
  }){
    Widget emptyErrorWidget = buildRefreshNoDataWidget(
        viewState: this,
        busyWidget: busyWidget,
        emptyDataWidget:
        emptyWidget,
        showErrorWidget:
        showErrorWidget,
        showEmptyWidget:
        showEmptyWidget,
        showBusyWidget: showBusyWidget
    );

    List<Widget> widgetList;
    Widget widget;
    bool useSliverWidget = false;

    if(isNotEmpty(widgetListBuilder)){
      if(isBusy || isError){
        widgetList = [
          SliverToBoxAdapter(
            child: Container(),
          )
        ];
      }else{
        widgetList = widgetListBuilder(data);
      }
    }else{
      if(isBusy || isError){
        widget = Container();
      }else{
        widget = widgetBuilder(data);
      }
    }


    if(isNotEmpty(widgetList)){
      useSliverWidget = true;
    }else{
      useSliverWidget = false;
    }

    if(isEmpty(footWidget)){
      return _buildEasyRefresh(
          scrollController: scrollController,
          useSliverWidget: useSliverWidget,
          emptyErrorWidget: emptyErrorWidget,
          onLoad: onLoad,
          onRefresh: onRefresh,
          widget: widget,
          widgetList: widgetList
      );
    }else{
      return Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          Expanded(
            child: _buildEasyRefresh(
                scrollController: scrollController,
                useSliverWidget: useSliverWidget,
                emptyErrorWidget: emptyErrorWidget,
                onLoad: onLoad,
                onRefresh: onRefresh,
                widget: widget,
                widgetList: widgetList
            ),
          ),
          footWidget??Container()
        ],
      );
    }
  }

  ///底层建立下拉刷新
  Widget _buildEasyRefresh({
    ScrollController scrollController,
    bool useSliverWidget,
    Widget emptyErrorWidget,
    Function onLoad,
    Function onRefresh,
    List<Widget> widgetList,
    Widget widget,
  }){
    return useSliverWidget? EasyRefresh.custom(
      slivers: widgetList,
      scrollController: scrollController,
      enableControlFinishRefresh: true,
      enableControlFinishLoad: true,
      taskIndependence: true,
      controller: refreshController,
      emptyWidget: emptyErrorWidget,
      onLoad: isBusy? null : isEmpty(onLoad)? null :  () async{
        onLoad.call();
      },
      onRefresh: isBusy? null : () async{
        onRefresh.call();
      },
    ) : EasyRefresh(
      enableControlFinishLoad: true,
      enableControlFinishRefresh: true,
      scrollController: scrollController,
      emptyWidget: emptyErrorWidget,
      taskIndependence: true,
      child: widget,
      controller: refreshController,
      onRefresh: isBusy? null : () async{
        onRefresh.call();
      },
    );
  }
}



/// 页面状态类型
enum ViewState {
  idle,
  busy,
  error
}