import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter_easyrefresh/easy_refresh.dart';
import 'package:flutter_mylibrary/http/exception/http_error.dart';
import '../../flutter_mylibrary.dart';
import '../base_view_state.dart';


abstract class ViewStateModel with ChangeNotifier, BaseViewState {


  /// 防止页面销毁后,异步任务才完成,导致报错
  bool _disposed = false;


  EasyRefreshController _refreshController = EasyRefreshController();

  set refreshController(EasyRefreshController easyRefreshController){
    this._refreshController = easyRefreshController;
  }

  void initData({bool retry = false});


  EasyRefreshController get refreshController => _refreshController;

  bool isInit = false;

  void getOtherData(){}


  set viewState(ViewState viewState) {
    this.viewState = viewState;
    notifyListeners();
  }


  /// set
  void setIdle() {
    viewState = ViewState.idle;
    isInit = true;
  }

  void setBusy() {
    viewState = ViewState.busy;
  }


  /// e为原始报错信息
  void setError(HttpError httpError) {

    viewState = ViewState.error;

    onError(httpError);
  }




  @override
  void notifyListeners() {
    if (!_disposed) {
      super.notifyListeners();
    }
  }

  @override
  void dispose() {
    _disposed = true;
    _refreshController.dispose();
    debugPrint('view_state_model dispose -->$runtimeType');
    _ViewModelStore._of().disRegister(this);
    super.dispose();
  }



  ///传入对应的ViewModel泛型
  ///[builder] 实例化view model
  ///
  /// 1. 取当前嵌套页面组件VM type数据 [ancestorOfType]=false
  /// 2. 取根页面组件VM type数据 [ancestorOfType]=true&&[builder]==null
  /// 3. 取当前页面存在VM type数据 [ancestorOfType]=false
  static VM of<VM extends ViewStateModel>({VM Function() builder, String instance, bool ancestorOfType = false}) {
    VM vm = _ViewModelStore._of().register<VM>(builder: builder, instanceName: instance);
    assert(vm != null, 'do you forget register the viewModel ${vm.runtimeType} in embed pager or top pager?');
    return vm;
  }


}

/// [e]为错误类型 :可能为 Error , Exception ,String
/// [s]为堆栈信息
printErrorStack(e, s) {
  debugPrint('''
<-----↓↓↓↓↓↓↓↓↓↓-----error-----↓↓↓↓↓↓↓↓↓↓----->
$e
<-----↑↑↑↑↑↑↑↑↑↑-----error-----↑↑↑↑↑↑↑↑↑↑----->''');
//   if (s != null) debugPrint('''
// <-----↓↓↓↓↓↓↓↓↓↓-----trace-----↓↓↓↓↓↓↓↓↓↓----->
// $s
// <-----↑↑↑↑↑↑↑↑↑↑-----trace-----↑↑↑↑↑↑↑↑↑↑----->
//     ''');
}



///内置一个provider管理者
class _ViewModelStore{

  ///用于存储状态
  final _factories = Map<Type, dynamic>();

  /// the ones that get registered by name.
  final _factoriesByName = Map<String, dynamic>();

  _ViewModelStore();

  static _ViewModelStore _viewModelStore;

  static _ViewModelStore _of(){
    if (_viewModelStore == null) {
      _viewModelStore = _ViewModelStore();
    }
    return _viewModelStore;
  }



  void disRegister(ViewStateModel viewStateModel){
    if (_factories.containsValue(viewStateModel)) {
      _factories.removeWhere((key, value) => value == viewStateModel);
    }else if (_factoriesByName.containsValue(viewStateModel)) {
      _factoriesByName.removeWhere((key, value) => value == viewStateModel);
    }
  }


  _clear() {
    _factories.values.forEach((value) {
      (value as ViewStateModel).dispose();
    });
    _factoriesByName.values.forEach((value) {
      (value as ViewStateModel).dispose();
    });
    _factories.clear();
    _factoriesByName.clear();
  }


  ///是否注册在provider管理器里
  bool isRegister<T extends ViewStateModel>({Type viewModelType, String instanceName}) {
    if (instanceName != null) {
      return _factoriesByName.containsKey(instanceName);
    } else if(viewModelType != null) {
      return _factories.containsKey(viewModelType);
    }
    return false;
  }


  ///泛型务必要加上
  ///如果不想使用该方法进行注册view model 请直接使用 [get]
  T register<T extends ViewStateModel>({String instanceName, @required T Function() builder}) {
    T viewModel;
    if (instanceName != null) {
      if (_factoriesByName.containsKey(instanceName)) {
        viewModel = _factoriesByName[instanceName];
        return viewModel;
      } else {
        viewModel = builder();
        debugPrint('register viewModel: $viewModel instanceName: $instanceName');
        _factoriesByName[instanceName] = viewModel;
        return viewModel;
      }
    } else {
      if (_factories.containsKey(T)) {
        viewModel = _factories[T];
        return viewModel;
      } else {
        viewModel = builder();
        debugPrint('register viewModel: $viewModel instanceName: $instanceName');
        _factories[T] = viewModel;
        return viewModel;
      }
    }
  }



}



